class Transporter {
  // { email: String }
  sendAuthorizationCode(args) {
    bridge.invoke('sendAuthorizationCode', args);

    return bridge.watchOne().promise;
  }

  // { code: String }
  authorize(args) {    
    bridge.invoke('authorize', args);

    // user:authorized | authorizeError
    return bridge.watchOne().promise;
  }

  // { agreedToTerms: Boolean }
  agreeToTerms(args, form) {   
    if (args.agreeToTerms) {
      bridge.invoke('agreeToTerms', args);
    }
    else {
      form.setValidationErrors([
        { key: 'agreeToTerms', message: 'You gotta agree to use Transporter.' }
      ]);
    }
  }

  renderComponent(args) {
    bridge.invoke('renderComponent', args);
    
    return bridge.watchOne().promise;
  }

  openDialog(args) {
    bridge.invoke('openDialog', { type: 'directory' });
  }

  open(args) {
    bridge.invoke('open', args);
  }

  deauthorize() {
    bridge.invoke('deauthorize', { });
  }

  quit() {
    bridge.invoke('quit');
  }

  syncUI(args) {    
    bridge.invoke('syncUI', { a: 1 });
  }

  sync(args) {    
    bridge.invoke('sync', { a: 1 });
  }

  // { url }
  navigate(args) {
    bridge.invoke('navigate', args);

    return bridge.watchOne().promise;
  }

  getUploadToken() {
    bridge.invoke('getUploadToken', { object: 'unknown' });
    
    return bridge.watchOne().promise;
  }

  commit(args, form) {
    let checkedEls = form.element.querySelectorAll("input:checked");
    let paths = [ ];

    for (var el of Array.from(checkedEls)) {
      var path = el.closest('li').dataset['key'];

      paths.push(path);
    }

    args.paths = paths;

    bridge.invoke('commitChanges', args);
  }


  checkRepository(args) {  
    bridge.invoke('checkRepository', args);
  
    return bridge.watchOne().promise;;
  }

  syncRepository(args) {  
    bridge.invoke('syncRepository', args);
  
    return bridge.watchOne().promise;;
  }

  fetch(args) {  
    bridge.invoke('fetch', args);
    
    return bridge.watchOne().promise;;
  }

  push(args) {  
    bridge.invoke('push', args);
  
    return bridge.watchOne().promise;;
  }

  pull(args) {  
    bridge.invoke('pull', args);
  
    return bridge.watchOne().promise;;
  }

  async initializeRepository(args, form) {  
    bridge.invoke('initializeRepository', args);
    
    await bridge.watchOne().promise;;

    new RepositoryComponent(args.id).render();
  }

  async handleFormSubmit(form) {
    let action = form.element.getAttribute('action');

    let args = form.serialize();

    form.element.classList.add('processing');

    if (action in this) {
      let result = this[action](args, form);

      if (typeof result === 'object' && 'then' in result) {
        result.then(message => {
          if (message.errors) {
            form.setValidationErrors(message.errors);
          }

          form.element.classList.remove('processing');
        }); 
      }
      else {
        form.element.classList.remove('processing');
      }
    }
    else {
      console.log('invoked default ƒ ' + action);

      bridge.invoke(action, args);
      
      var message = await bridge.watchOne().promise;

      form.element.classList.remove('processing');


      if (message.errors) {
        form.setValidationErrors(message.errors);
      }
    }
  }
}

var app = new Transporter();

console.log('app', app);

document.body.addEventListener('carbon:drop', async (e) => {
  console.log('dropped file', e);  
  
  var token = await app.getUploadToken();

  let upload = new Carbon.Upload(e.detail.files[0], { 
    url           : token.url,
    authorization : 'Bearer ' + token.token,
    chuckSize     : 1024 * 1024 * 64 // 64MB chunks
  });

  var result = await upload.start();
  
  bridge.invoke('updateProfile', {
    id      : 0, 
    imageId : result.id
  });
});









var elementControllers = new WeakMap(); // element | instances

class CollaborationComponent {
  static get(id) {
    let element = document.querySelector('#collaboration_' + id);
    
    return elementControllers.get(element) || new CollaborationComponent(element);
  }

  constructor(element) {
    this.element = element;
    this.id = element.dataset['id'];

    elementControllers.set(element, this);
  }

  accept() {
    this.element.classList.add('accepted');
    this.element.classList.add('saved');

    bridge.invoke('acceptCollaboration', { id: this.id });
  }

  reject() {
    this.element.classList.add('rejected');
    this.element.classList.add('saved');

    bridge.invoke('rejectCollaboration', { id: this.id });
  }

  save() {
    // TODO: Working directory
    this.element.classList.remove('accepted');
    this.element.classList.add('saved');

    bridge.invoke('rejectCollaboration', { id: this.id });

  }
}

class EnvironmentComponent {

}





async function move() { 
  var d = 1;

  for (var i = 0; i < 500; i++) {
    bridge.invoke('move', { x: d * i, y: d * i });
    
    console.log('hello');

    await delay(10);

  }

  
};

function delay(t) { 
  return new Promise(function(resolve) { 
    setTimeout(resolve, t)
  });
}

