// - Footer

class TopBar {

  constructor(element) {
    this.element = element;
    
    
    this.element.addEventListener('mousedown', this.onMouseDown.bind(this));
  }

  onMouseDown(e) {
    this.origin = e;
    
    this.isDragging = true;

    this.moveListener = this.mouseMove.bind(this);

    window.addEventListener('mousemove', this.moveListener);
  
    window.addEventListener('mouseup', () => {
      this.isDown = false;

      window.removeEventListener('mousemove', this.moveListener);

      if (this.last) {
        app.window.x = this.last.x;
        app.window.y = this.last.y;
      }
    });
  }

  mouseMove(e) {
    if (!this.isDragging) {
      return;
    }

    var deltaX = this.origin.screenX - e.screenX;
    var deltaY = this.origin.screenY - e.screenY;

    var newPosition = {
      x: app.window.x - deltaX,
      y: app.window.y + deltaY
    };

    // if (newPosition.y < 0 || newPosition.x < 0) return;


    bridge.invoke('move', newPosition);

    this.last = newPosition;
  }
}

var top = new TopBar(document.querySelector('.topBar'));

class Sidebar {
  constructor() {
    this.element = document.querySelector('.sidebar');
  }

  async render() {
    console.log('render sidebar 1');

    var a = await app.renderComponent({
      name    : 'Sidebar',
      id      : 'sidebar'
    });
    
    let el = getElement(a.body);

    this.element.replaceWith(el);

    this.element = document.querySelector('.sidebar');
  }

  select(el) {
    var id = el.dataset['id'];

    bridge.invoke('selectOrganization', { id: id });
    
    app.navigate({ url: '/organization' });

    
    for (var e of this.element.querySelectorAll('.selected'))
    {
      e.classList.remove('selected');
    }

    el.classList.add('selected');



  }
}

function selectRepository(id) {

  bridge.invoke('selectRepository', { id: id });
    
  app.navigate({ url: '/repository' });
}

app.sidebar = new Sidebar();


class UserComponent {
  constructor(element) {
    this.element = element;
  
    this.model = { name: '', image: { url: '' } };
  }

  update(model) {

  }
}

reactive.on('object:changed', e => {
  console.log(e.object);
  
  if (e.object.repositoryId) {
    new RepositoryComponent(e.object.repositoryId).render();
  }

});

reactive.on('changes:commited', e => {
  console.log('commited changes', e.repository);
  
  if (e.repository.id) {
    new RepositoryComponent(e.repository.id).render();
  }

});


class RepositoryComponent {
  constructor(id) {
    this.id = id;
    this.element = document.getElementById('repository_' + id);
  }
  
  async sync() {
    console.log('sync');

    document.getElementById('syncButton').innerText = 'syncing';

    await app.syncRepository({ id: this.id });

    document.getElementById('syncButton').innerText = 'synced';
  }

  async push(e) {
    e.innerHTML = '...';

    await app.push({ id: this.id });

    app.navigate({ url: '/repository' });
  }

  async pull(e) {
    e.innerHTML = '...';

    await app.pull({ id: this.id });

    app.navigate({ url: '/repository' });
  }


  async fetch(e) {
    console.log('fetch', e);

    e.innerHTML = 'fetching...';

    await app.fetch({ id: this.id });

    e.innerHTML = 'done';

    app.navigate({ url: '/repository' });
  }

  async check() {
    await app.checkRepository({ id: this.id });
  
    await this.render();
  }

  async render() { 
    if (!this.element) return;

    console.log('reload repository:' + this.id);

    var a = await app.renderComponent({
      name    : 'Changeset',
      id      : 'changeset_' + this.id,
      object  : 'repositories/' + this.id
    });

    var el = getElement(a.body);

    this.element.querySelector('.changeset').replaceWith(el);

    onDomChange();
  }

}

reactive.on('user:updated', async e => {
  app.sidebard.render();
});


function getElement(html) {
  var template = document.createElement('template');
  html = html.trim();
  template.innerHTML = html;
  return template.content.firstChild;
}


// - Activities

class ActivityComponent {
  constructor(element) {
    this.element = element;
  }
}


class ChangeComponent {
  constructor(element) {
    this.element = element;
  }
}